      program extract
C----
C---- (C) Bert de Groot, Goettingen 2000
C----
C---- read the output from 'rottrans' and generates coordinate
C---- files for the best-fitting fragments
C----      
      implicit none

      include 'params.f'

      integer i,l,nat,atnr(maxat),resnr(maxat),nres,a,
     &     nx,ny,nz,ngrid(3),orig(3),ntit,nstr,iargc,gvflen
      
      character*5 atnam(maxat),resnam(maxat),dum1

      character*4 cdum1,cdum2

      character*1 chainid(maxat)

      real pi,xx(maxat),yy(maxat),zz(maxat),xav(3),xr(maxat),
     &     yr(maxat),zr(maxat),alpha,beta,gamma,ri,rj,rk,ru,rv,rw,
     &     ca,sa,cb,sb,cg,sg,space(3),cell(6),score,
     &     Bfac(maxat),occ(maxat)

      character*80 sdum

      character*255 buf

      logical gvfisf
      
      if (iargc().ne.3) then
         write(0,'(a,a)') 'Usage: extract in.map in.scores ',
     &        'nr_of_structures < in.pdb'
         write(0,*) 
         write(0,'(a,a)') 'It reads output from "rottrans" ',
     &        '("in.scores") and generates corresponding'
         write(0,'(a,a)') 'coordinate files. Please use the same ',
     &        '"in.map" and "in.pdb" as in "rottrans".'
         write(0,'(a,a)') '"in.map" has to be x-plor formatted format',
     &        ' for now.'
         stop
      endif

      xav(1)=0.0
      xav(2)=0.0
      xav(3)=0.0
      pi=4.0*atan(1.0)
C----
C---- read input structure
C----
      nat=0
      nres=1
      cdum2='xxxx'
 50   continue
      read(5,'(a80)',err=60,end=70) sdum
      if ((sdum(1:4).ne.'ATOM').and.(sdum(1:4).ne.'HETA')) goto 50
      nat=nat+1
      if (nat.gt.maxat) then
         write(0,'(a)') 'Error - MAXAT exceeded'
         stop
      endif
      read(sdum,2000) atnam(nat),resnam(nat),chainid(nat),cdum1,
     &     xx(nat),yy(nat),zz(nat),occ(nat),Bfac(nat)
      if (nat.gt.1) then
         if (cdum1.ne.cdum2) nres=nres+1
      endif
      if (atnam(nat).ne.'     ') then
         call gvsshr(atnam(nat),5)
      else
         write(0,*) 'Encountered empty atom name for atom number',nat
         stop
      endif
      resnr(nat)=nres
      atnr(nat)=nat
      cdum2=cdum1
      goto 50
 60   write(0,'(a)') 'Error while reading input structure from stdin'
      stop
 70   continue
      if (nat.eq.0) then
         write(0,'(a,a)') 'Unexpected end of file while reading ',
     &        'input structure from stdin'
         stop
      endif
      write(6,'(a,i5)') 'Number of atoms:',nat

      do i=1,nat
         xav(1)=xav(1)+xx(i)
         xav(2)=xav(2)+yy(i)
         xav(3)=xav(3)+zz(i)
      enddo

      xav(1)=xav(1)/real(nat)
      xav(2)=xav(2)/real(nat)
      xav(3)=xav(3)/real(nat)

      do i=1,nat
         xx(i)=xx(i)-xav(1)
         yy(i)=yy(i)-xav(2)
         zz(i)=zz(i)-xav(3)
      enddo
C----
C---- read map header for params
C----
      call getarg(1,sdum)
      call readxplormapheader(sdum,cell,nx,ny,nz,space,orig,ngrid)
C----
C---- read the highest scoring orientations from file and generate them
C----
      call getarg(2,sdum)
      if (gvfisf(sdum)) then
         open(1,file=sdum,status='old')
      else
         call gvstcp(sdum)
         write(0,'(a,a)') 'Error - file not found : ',
     &        sdum(1:gvflen(sdum))
         stop
      endif
      call getarg(3,sdum)
      read(sdum,*) nstr
      do a=1,nstr
         read(1,*,end=100) ri,rj,rk,ru,rv,rw,score
         write(sdum,'(a,i8,a)') 'extract',a,'.pdb'
         call gvstcp(sdum)
         open(2,file=sdum,status='unknown')
         write (2,'(a)') 'REMARKS Structure written by extract'
         write (2,'(a,f10.5)') 'REMARKS Score = ',score
         write (2,'(a,3f10.5)') 'REMARKS Rotations = ',ri,rj,rk
         write (2,'(a,3f10.5)') 'REMARKS Translations = ',ru,rv,rw
         gamma=pi*ri/180.0
         cg=cos(gamma)
         sg=sin(gamma)
         beta=pi*rj/180.0
         cb=cos(beta)
         sb=sin(beta)
         alpha=pi*rk/180.0
         ca=cos(alpha)
         sa=sin(alpha)
         do l=1,nat
            xr(l)=xx(l)*(cg*cb*ca-sg*sa)
            xr(l)=xr(l)-yy(l)*(sg*cb*ca+cg*sa)
            xr(l)=xr(l)+zz(l)*(sb*ca)
            xr(l)=xr(l)+(ru+real(orig(1)-1))*space(1)
            yr(l)=xx(l)*(cg*cb*sa+sg*ca)
            yr(l)=yr(l)-yy(l)*(sg*cb*sa-cg*ca)
            yr(l)=yr(l)+zz(l)*(sb*sa)
            yr(l)=yr(l)+(rv+real(orig(2)-1))*space(2)
            zr(l)=xx(l)*(-cg*sb)
            zr(l)=zr(l)+yy(l)*(sg*sb)
            zr(l)=zr(l)+zz(l)*cb
            zr(l)=zr(l)+(rw+real(orig(3)-1))*space(3)
            dum1=atnam(l)
            call gvsshl(dum1)
            if (gvflen(dum1).lt.4) then
               write(cdum1,'(a1,a3)') ' ',dum1(1:3)
            else
               cdum1=dum1(1:4)
            endif
            write (2,1000) l,cdum1,resnam(l)(1:3),chainid(l),
     &        resnr(l),xr(l),yr(l),zr(l),10.*score,Bfac(l)
         enddo
         write (2,'(a)') 'TER'
         close (2)
      enddo

 100  continue
               
      close(1)

 1000 format ('ATOM  ',i5,1x,a4,1x,a3,1x,a1,i4,4x,3f8.3,2f6.2)
 2000 format (12x,a4,1x,a4,a1,a4,4x,3f8.3,2f6.2)

      end
      
