      subroutine calc_dens(den1,den2,id,a1,a2,b1,b2,x,y,z,Bfac,nat,
     &     calden,space,orig,nx,ny,nz,maxat,maxmap)
C----
C---- calculates densities from coordinates
C---- this is the most simple, basic form. See below and in Concoord
C---- for more specialised and more efficient forms.
C----
      implicit none

      integer maxmap,maxat

      integer i,j,k,l,nat,id(*),imin,imax,jmin,jmax,kmin,kmax,
     &     margin,nx,ny,nz,orig(*),notinmap,dim,dim2,ind
      
      real den1(*),a1(*),a2(*),b1(*),b2(*),pi,calden(*),
     &     minfourpisq,crit,x(*),y(*),z(*),dis,den2(*),
     &     xd(3),xx,yy,zz,space(*),contrib,cutoff,Bfac(*),bd1,bd2

      logical inmap
C----
C---- initialise
C---- cutoff sets the radius (in A) around each atom that is taken into account
C---- 2.5 A was empirically derived as a satisfactory compromise between
C---- accuracy and performance
C----
      dim=nx
      dim2=nx*ny
      cutoff=2.5
      margin=int(cutoff/min(space(1),space(2),space(3)))
      crit=cutoff**2
      pi=4.0*atan(1.0)
      minfourpisq=-(4.*pi*pi)
      do i=1,nx*ny*nz
         calden(i)=0.0
      enddo
c----
c---- check if atom coordinates are inside map volume
c---- 
      notinmap=0
      do l=1,nat
         inmap=.true.
         i=nint(x(l)/space(1))-orig(1)+1
         if (i.lt.1.or.i.gt.nx) inmap=.false.
         i=nint(y(l)/space(2))-orig(2)+1
         if (i.lt.1.or.i.gt.ny) inmap=.false.
         i=nint(z(l)/space(3))-orig(3)+1
         if (i.lt.1.or.i.gt.nz) inmap=.false.
         if (.not.inmap) notinmap=notinmap+1
      enddo
      if (notinmap.eq.nat) then
         write(0,'(a,a)') 'Error: atom coordinates do not ',
     &        'match map'
         stop
      endif
      if (notinmap.gt.0) then
         write(0,'(a,i5)') 'Warning : nr of atoms outside map:',
     &        notinmap
      endif
C----
C---- create densities using the map parameters
C----
      do l=1,nat
         imin=nint(x(l)/space(1))-margin-orig(1)+1
         imin=max(1,imin)
         imax=nint(x(l)/space(1))+margin-orig(1)+1
         imax=min(nx,imax)
         jmin=nint(y(l)/space(2))-margin-orig(2)+1
         jmin=max(1,jmin)
         jmax=nint(y(l)/space(2))+margin-orig(2)+1
         jmax=min(ny,jmax)
         kmin=nint(z(l)/space(3))-margin-orig(3)+1
         kmin=max(1,kmin)
         kmax=nint(z(l)/space(3))+margin-orig(3)+1
         kmax=min(nz,kmax)
c         write(0,*) imin,imax,jmin,jmax,kmin,kmax
         xx=x(l)
         yy=y(l)
         zz=z(l)
         bd1=b1(id(l))+Bfac(l)
         bd2=b2(id(l))+Bfac(l)
         do i=imin,imax
            do j=jmin,jmax
               do k=kmin,kmax
                  xd(1)=real(i+orig(1)-1)*space(1)
                  xd(2)=real(j+orig(2)-1)*space(2)
                  xd(3)=real(k+orig(3)-1)*space(3)
                  dis=(xd(1)-xx)**2
                  dis=dis+(xd(2)-yy)**2
                  dis=dis+(xd(3)-zz)**2
                  if (dis.lt.crit) then
                     contrib=den1(l)*exp(minfourpisq*dis/bd1)+
     &                    den2(l)*exp(minfourpisq*dis/bd2)
                     ind=(k-1)*dim2+(j-1)*dim+i
                     calden(ind)=calden(ind)+contrib
                  endif
               enddo
            enddo
         enddo
      enddo

      return

      end

      subroutine calc_dens2(den1,den2,id,a1,a2,b1,b2,x,y,z,cutoff,nat,
     &     imin,imax,jmin,jmax,kmin,kmax,np,ppi,ppj,ppk,ppd,
     &     avcal,sigcal,space,nx,ny,nz,Bfac)

      implicit none

      include 'params.f'

      integer i,j,k,l,n,nat,id(*),np,ppi(*),
     &     imin,imax,jmin,jmax,kmin,kmax,ppj(*),ppk(*),
     &     margin,dx,dy,dz,nx,ny,nz,iimin,iimax,jjmin,jjmax,
     &     kkmin,kkmax,dim,dim2
      
      real den1(*),a1(*),a2(*),b1(*),b2(*),pi,calden(maxmap),x(*),y(*),
     &     z(*),dis,den2(*),xd(3),minfourpisq,xx,yy,zz,ppd(*),space(*),
     &     contrib,cutoff,crit,avcal,sigcal,dd1,dd2,db1,db2,Bfac(*)

      logical bfirst

      data bfirst /.true./

      save bfirst
      save calden
C----
C---- initialise
C----
      margin=int(cutoff/min(space(1),space(2),space(3)))
      crit=cutoff**2
      pi=4.0*atan(1.0)
      minfourpisq=-(4.*pi*pi)
      dx=(nx-1)/2
      dy=(ny-1)/2
      dz=(nz-1)/2
      dim=nx
      dim2=nx*ny
      if (nx*ny*nz.gt.maxmap) then
         write(0,'(a)') 'Error -  MAXMAP exceeded in calc_dens'
         stop
      endif

      if (bfirst) then
         do i=1,nx*ny*nz
            calden(i)=0.0
         enddo
         bfirst=.false.
      endif
C----
C---- create densities around the center of a temporary map
C----
      iimin=1000
      jjmin=1000
      kkmin=1000
      iimax=-1000
      jjmax=-1000
      kkmax=-1000
      do l=1,nat
         imin=nint(x(l)/space(1))+dx-margin
         imin=max(1,imin)
         imax=nint(x(l)/space(1))+dx+margin
         imax=min(nx,imax)
         jmin=nint(y(l)/space(2))+dy-margin
         jmin=max(1,jmin)
         jmax=nint(y(l)/space(2))+dy+margin
         jmax=min(ny,jmax)
         kmin=nint(z(l)/space(3))+dz-margin
         kmin=max(1,kmin)
         kmax=nint(z(l)/space(3))+dz+margin
         kmax=min(nz,kmax)
         xx=x(l)
         yy=y(l)
         zz=z(l)
         dd1=den1(l)
         dd2=den2(l)
         db1=b1(id(l))+Bfac(l)
         db2=b2(id(l))+Bfac(l)
         do k=kmin,kmax
            do j=jmin,jmax
               do i=imin,imax
                  xd(1)=real(i-dx)*space(1)
                  xd(2)=real(j-dy)*space(2)
                  xd(3)=real(k-dz)*space(3)
                  dis=(xd(1)-xx)**2
                  dis=dis+(xd(2)-yy)**2
                  dis=dis+(xd(3)-zz)**2
                  if (dis.lt.crit) then
                     dis=minfourpisq*dis
                     contrib=dd1*exp(dis/db1)+dd2*exp(dis/db2)
                     iimin=min(iimin,i)
                     jjmin=min(jjmin,j)
                     kkmin=min(kkmin,k)
                     iimax=max(iimax,i)
                     jjmax=max(jjmax,j)
                     kkmax=max(kkmax,k)
                     n=(k-1)*dim2+(j-1)*dim+i
                     calden(n)=calden(n)+contrib
                  endif
               enddo
            enddo
         enddo
      enddo
C----
C---- make list of tagged grid points with significant model density
C----
      np=0
      avcal=0.0
      sigcal=0.0
      do k=kkmin,kkmax
         do j=jjmin,jjmax
            do i=iimin,iimax
               n=(k-1)*dim2+(j-1)*dim+i
               if (calden(n).gt.1.0e-5) then
                  np=np+1
                  if (np.gt.maxgrid) then
                     write(0,'(a)') 'Error - MAXGRID exceeded'
                     stop
                  endif
                  ppi(np)=i-dx
                  ppj(np)=j-dy
                  ppk(np)=k-dz
                  ppd(np)=calden(n)
                  avcal=avcal+ppd(np)
                  sigcal=sigcal+ppd(np)**2
               endif
               calden(n)=0.0
            enddo
         enddo
      enddo
      imin=iimin-dx
      imax=iimax-dx
      jmin=jjmin-dy
      jmax=jjmax-dy
      kmin=kkmin-dz
      kmax=kkmax-dz

      avcal=avcal/float(np)
      sigcal=sqrt(sigcal/float(np) - avcal**2)
c      do i=1,np
c         ppd(i)=(ppd(i)-avcal)/sigcal
c      enddo


c      do i=1,np
c         write(6,*) ppi(i),ppj(i),ppk(i),ppd(i)
c      enddo

c      write(6,*) imin,imax,jmin,jmax,kmin,kmax

c      write(6,*) 'np=',np,' ntot=',((kmax-kmin)+1)*((jmax-jmin)+1)*
c     &     ((imax-imin)+1)
         
      return

      end

      subroutine calc_dens3(den1,den2,id,a1,a2,b1,b2,x,y,z,nat,
     &     imin,imax,jmin,jmax,kmin,kmax,np,calden,space,orig,nx,ny,nz,
     &     maxat,maxmap,maxgrid,ppi,ppj,ppk,ppc,occ,Bfac,scale,shift)
C----
C---- calculates densities from coordinates
C----
      implicit none

      integer maxmap,maxat,maxgrid

      integer i,j,k,l,n,nat,id(*),np,imin,imax,jmin,jmax,kmin,kmax,
     &     ind,margin,nx,ny,nz,iimin,iimax,jjmin,ppi(*),ppj(*),ppk(*),
     &     jjmax,kkmin,kkmax,orig(*),notinmap,ind3,dim,dim2
      
      real den1(*),a1(*),a2(*),b1(*),b2(*),pi,Bfac(*),db1,db2,
     &     calden(*),minfourpisq,crit,scale,shift,rdum,
     &     x(*),y(*),z(*),dis,den2(*),xd,yd,zd,occ(*),
     &     xx,yy,zz,space(*),contrib,cutoff,ppc(maxgrid,*)

      logical inmap
C----
C---- initialise
C---- cutoff sets the radius (in A) around each atom that is taken into account
C---- 2.0 A was empirically derived as a satisfactory compromise between
C---- accuracy and performance
C----
      dim=nx
      dim2=nx*ny
      cutoff=2.5
      margin=int(cutoff/min(space(1),space(2),space(3)))
      crit=cutoff**2
      pi=4.0*atan(1.0)
      minfourpisq=-(4.*pi*pi)

      do n=1,np
         i=ppi(n)
         j=ppj(n)
         k=ppk(n)
         ind=ind3(i,j,k,dim,dim2)
      enddo

      do i=1,nx*ny*nz
         calden(i)=0.0
      enddo
c---- 
c---- check if atom coordinates are inside map volume
c---- 
      notinmap=0
      do l=1,nat
         inmap=.true.
         i=nint(x(l)/space(1))-orig(1)+1
         if (i.lt.1.or.i.gt.nx) inmap=.false.
         i=nint(y(l)/space(2))-orig(2)+1
         if (i.lt.1.or.i.gt.ny) inmap=.false.
         i=nint(z(l)/space(3))-orig(3)+1
         if (i.lt.1.or.i.gt.nz) inmap=.false.
         if (.not.inmap) notinmap=notinmap+1
      enddo
      if (notinmap.eq.nat) then
         write(0,'(a,a)') 'Error: atom coordinates do not ',
     &        'match map'
         stop
      endif
      if (notinmap.gt.0) then
         write(0,'(a,i5)') 'Warning : nr of atoms outside map:',
     &        notinmap
      endif
C----
C---- create densities using the map parameters
C----
      iimin=1000
      jjmin=1000
      kkmin=1000
      iimax=-1000
      jjmax=-1000
      kkmax=-1000
      do l=1,nat
         imin=nint(x(l)/space(1))-margin-orig(1)+1
         imin=max(1,imin)
         imax=nint(x(l)/space(1))+margin-orig(1)+1
         imax=min(nx,imax)
         jmin=nint(y(l)/space(2))-margin-orig(2)+1
         jmin=max(1,jmin)
         jmax=nint(y(l)/space(2))+margin-orig(2)+1
         jmax=min(ny,jmax)
         kmin=nint(z(l)/space(3))-margin-orig(3)+1
         kmin=max(1,kmin)
         kmax=nint(z(l)/space(3))+margin-orig(3)+1
         kmax=min(nz,kmax)
c         write(31,*) imin,imax,jmin,jmax,kmin,kmax
         xx=x(l)
         yy=y(l)
         zz=z(l)
         db1=b1(id(l))+Bfac(l)
         db2=b2(id(l))+Bfac(l)
         do k=kmin,kmax
            do j=jmin,jmax
               do i=imin,imax
                  xd=real(i+orig(1)-1)*space(1)
                  yd=real(j+orig(2)-1)*space(2)
                  zd=real(k+orig(3)-1)*space(3)
                  dis=(xd-xx)**2
                  dis=dis+(yd-yy)**2
                  dis=dis+(zd-zz)**2
                  if (dis.lt.crit) then
                     contrib=den1(l)*exp(minfourpisq*dis/db1)+
     &                    den2(l)*exp(minfourpisq*dis/db2)
c                     write(33,*) sqrt(dis),contrib
                     iimin=min(iimin,i)
                     jjmin=min(jjmin,j)
                     kkmin=min(kkmin,k)
                     iimax=max(iimax,i)
                     jjmax=max(jjmax,j)
                     kkmax=max(kkmax,k)
                     ind=ind3(i,j,k,dim,dim2)
                     calden(ind)=calden(ind)+contrib
                  endif
               enddo
            enddo
         enddo
      enddo
C----
C----calculate statistics
C----
      np=0
#     if defined (OPTIMISE_OCCUPANCIES)
      do i=1,maxgrid
         do j=1,nat
            ppc(i,j)=0.0
         enddo
      enddo
#     endif
      do k=kkmin,kkmax
         do j=jjmin,jjmax
            do i=iimin,iimax
               ind=ind3(i,j,k,dim,dim2)
               if (calden(ind).gt.1.0e-5) then
                  np=np+1
                  if (np.gt.maxgrid) then
                     write(0,'(a)') 'MAXGRID exceeded'
                     stop
                  endif
                  ppi(np)=i
                  ppj(np)=j
                  ppk(np)=k
#     if defined (OPTIMISE_OCCUPANCIES)
                  rdum=0.0
                  do l=1,nat
                     db1=b1(id(l))+Bfac(l)
                     db2=b2(id(l))+Bfac(l)
                     xd=real(i+orig(1)-1)*space(1)
                     yd=real(j+orig(2)-1)*space(2)
                     zd=real(k+orig(3)-1)*space(3)
                     dis=(xd-x(l))**2
                     dis=dis+(yd-y(l))**2
                     dis=dis+(zd-z(l))**2
                     if (dis.lt.crit) then
                        ppc(np,l)=den1(l)*
     &                       exp(minfourpisq*dis/db1)+
     &                       den2(l)*exp(minfourpisq*dis/db2)
                        ppc(np,l)=ppc(np,l)*scale+shift*(ppc(np,l)/
     &                       calden(ind))
                        ppc(np,l)=ppc(np,l)*occ(l)
                        rdum=rdum+ppc(np,l)
                     endif
                  enddo
                  calden(ind)=rdum
#     endif
               else
                  calden(ind)=0.0
               endif
            enddo
         enddo
      enddo

      imin=iimin
      imax=iimax
      jmin=jjmin
      jmax=jjmax
      kmin=kkmin
      kmax=kkmax

      return

      end

      subroutine getatomscat(a1,a2,b1,b2,atnam,nat,Bfac,id,den1,den2)

      implicit none

      integer i,j,id(*),nscat,nat,line

      real a1(*),a2(*),b1(*),b2(*),den1(*),den2(*),Bfac(*),pi

      character*5 atnam(*),atdum

      character*1 adum(10),aa

      character*80 libdir

      logical bfound

      pi=4.0*atan(1.0)
      line=1
      call getenv('MAPLIB',libdir)
      call bbopen(1,libdir,'atscat.dat','old')
      read(1,*,err=100,end=100) nscat
      line=line+1
      read(1,*,err=100,end=100)
      line=line+1
      write(6,'(a)') 'Atomic scattering factors:'
      do i=1,nscat
         read(1,'(2x,a1)',err=100,end=100) adum(i)
         line=line+1
         read(1,*,err=100,end=100) a1(i),b1(i),a2(i),b2(i)
         write(6,'(i3,a2,4f8.3)') i,adum(i),a1(i),b1(i),a2(i),b2(i)
         line=line+1
         b1(i)=b1(i)
         b2(i)=b2(i)
      enddo
      goto 200
 100  write(0,'(a)') 'Error while reading atomic scattering data from:'
      write(0,'(a,a)') 'atscat.dat in ',libdir
      write(0,'(a,i5)') 'line:', line
      stop
 200  continue
      close(1)
C----
C---- assign scattering types to atoms
C----
      do i=1,nat
         atdum=atnam(i)
         call gvstcp(atdum)
         aa=atdum(1:1)
         bfound=.false.
         do j=1,nscat
            if (aa.eq.adum(j)) then
               id(i)=j
               bfound=.true.
            endif
         enddo
         if (.not.bfound) then
            write(0,'(a,a,i5,a5)') 'Error - unknown atom name for ', 
     &           'atom ',i,atnam(i)
            stop
         endif
      enddo
C----
C---- scattering prefactors
C----
      do i=1,nat
         den1(i)=a1(id(i))*(4.0*pi/(b1(id(i))+Bfac(i)))**1.5
         den2(i)=a2(id(i))*(4.0*pi/(b2(id(i))+Bfac(i)))**1.5
      enddo 
      
      return
      
      end

      subroutine bbopen(unit,dirnm,filnm,status)

      implicit none

      integer unit,len1,len2,gvflen

      character*(*) dirnm,filnm,status

      character*80 name

      logical gvfisf

      call gvsshl(dirnm)
      len1=gvflen(dirnm)
      call gvsshl(filnm)
      len2=gvflen(filnm)
      if (len1+len2.gt.79) then
         write(0,300)
         write(0,*) 'dirnm=',dirnm(1:len1),'with length ' ,len1
         write(0,*) 'filnm=',filnm(1:len2),'with length ',len2
         stop
      endif
      write(name,110) dirnm(1:len1),'/',filnm(1:len2)
C----
C---- first check current directory, then libdir
C----
      if (gvfisf(filnm)) then
         open(unit,file=filnm,status=status)
      else if (gvfisf(name)) then
         open(unit,file=name,status=status)
      else
         write(0,200) filnm(1:len2)
         write(0,210)
         stop
      endif

 110  format(a,a1,a)
 200  format('ERROR - file not found: ',a)
 210  format('please check MAPLIB environment variable')
 300  format('ERROR - buffer size exceeded in subroutine bbopen')

      return

      end

      subroutine update_scores(n,ri,rj,rk,u,v,w,score,rscore,
     &     maxscore)

      implicit none
      
      integer maxscore

      integer u,v,w,n

      real score,rscore(7,*),ri,rj,rk

      if (score.gt.rscore(7,n)) then
         rscore(1,n)=ri
         rscore(2,n)=rj
         rscore(3,n)=rk
         rscore(4,n)=real(u)
         rscore(5,n)=real(v)
         rscore(6,n)=real(w)
         rscore(7,n)=score
      endif

      return

      end

      subroutine update_n_scores(ri,rj,rk,u,v,w,score,rscore,
     &     maxscore)

      implicit none
      
      integer maxscore

      integer u,v,w,n,nn

      real score,rscore(7,*),ri,rj,rk

      do n=1,maxscore
         if (score.gt.rscore(7,n)) then
            do nn=maxscore+1,n+1,-1
               rscore(1,nn)=rscore(1,nn-1)
               rscore(2,nn)=rscore(2,nn-1)
               rscore(3,nn)=rscore(3,nn-1)
               rscore(4,nn)=rscore(4,nn-1)
               rscore(5,nn)=rscore(5,nn-1)
               rscore(6,nn)=rscore(6,nn-1)
               rscore(7,nn)=rscore(7,nn-1)
            enddo
            rscore(1,n)=ri
            rscore(2,n)=rj
            rscore(3,n)=rk
            rscore(4,n)=real(u)
            rscore(5,n)=real(v)
            rscore(6,n)=real(w)
            rscore(7,n)=score
            return
         endif
      enddo

      return

      end
      subroutine getscale(calden,expden,imin,imax,jmin,jmax,kmin,kmax,
     &     shift,scale,maxmap,dim,dim2)

C----
C---- compute the optimal scaling parameters to be applied to the calculated
C---- densities in comparison to the experimental densities (in the least
C---- squares sense)
C----
C---- obsolete, please use getscale2
C----
      implicit none

      integer i,j,k,imin,imax,jmin,jmax,kmin,kmax,maxmap,ind,ind3,
     &     dim,dim2,ndata

      real calden(maxmap),expden(maxmap),shift,scale,rdum

      real*8 serr,a,b,xs,ys,x2s,xys,delta
C----
C---- perform linear regression: y=a+bx (y=expden, x=calden)
C---- slope yields scale, intercept gives shift
C----
      xs=0.0
      ys=0.0
      x2s=0.0
      xys=0.0
      ndata=0
      do k=kmin,kmax
         do j=jmin,jmax
            do i=imin,imax
               ind=ind3(i,j,k,dim,dim2)
               if (calden(ind).gt.1.0e-5) then
                  ndata=ndata+1
                  xs=xs+calden(ind)
                  ys=ys+expden(ind)
                  x2s=x2s+calden(ind)**2
                  xys=xys+calden(ind)*expden(ind)
               endif
            enddo
         enddo
      enddo
      delta=dble(ndata)*x2s-xs**2
      a=(x2s*ys-xs*xys)/delta
      b=(dble(ndata)*xys-xs*ys)/delta
      
      shift=real(a)
      scale=real(b)
C----
C---- calculate the sum of squared errors
C----
      serr=0.0
      do k=kmin,kmax
         do j=jmin,jmax
            do i=imin,imax
               ind=ind3(i,j,k,dim,dim2)
               if (calden(ind).gt.1.0e-5) then
                  rdum=calden(ind)*scale+shift
                  serr=serr+(expden(ind)-rdum)**2
               endif
            enddo
         enddo
      enddo
      write(6,'(a,3e14.5)') 'First   shift,scale,error :',shift,
     &     scale,serr

c      return
C----
C---- perform linear regression: y=a+bx (y=calden, x=expden)
C---- slope yields scale, intercept gives shift
C----
      xs=0.0
      ys=0.0
      x2s=0.0
      xys=0.0
      ndata=0
      do k=kmin,kmax
         do j=jmin,jmax
            do i=imin,imax
               ind=ind3(i,j,k,dim,dim2)
               if (calden(ind).gt.1.0e-5) then
                  ndata=ndata+1
                  xs=xs+expden(ind)
                  ys=ys+calden(ind)
                  x2s=x2s+expden(ind)**2
                  xys=xys+calden(ind)*expden(ind)
               endif
            enddo
         enddo
      enddo
      delta=dble(ndata)*x2s-xs**2
      a=(x2s*ys-xs*xys)/delta
      b=(dble(ndata)*xys-xs*ys)/delta
C----
C---- calculate the sum of squared errors
C----
      serr=0.0
      ys=0.0
      do k=kmin,kmax
         do j=jmin,jmax
            do i=imin,imax
               ind=ind3(i,j,k,dim,dim2)
               if (calden(ind).gt.1.0e-5) then
                  ys=ys+calden(ind)*scale
                  rdum=(calden(ind)-a)/b
                  serr=serr+(expden(ind)-rdum)**2
               endif
            enddo
         enddo
      enddo
      write(6,'(a,3e14.5)') 'Second  shift,scale,error :',0.0-a,
     &     1.0/b,serr

      scale=1.0/b
      shift=scale*(0.0-a)
      return
C----
C---- calculate the 'symmetric' parameters
C----
      scale=(scale+1.0/real(b))/2.0
      shift=(xs-ys)/real(ndata)
C----
C---- calculate the sum of squared errors
C----
      serr=0.0
      do k=kmin,kmax
         do j=jmin,jmax
            do i=imin,imax
               ind=ind3(i,j,k,dim,dim2)
               if (calden(ind).gt.1.0e-5) then
                  rdum=calden(ind)*scale+shift
                  serr=serr+(expden(ind)-rdum)**2
               endif
            enddo
         enddo
      enddo
      write(6,'(a,3e14.5)') 'Optimal shift,scale,error :',shift,
     &     scale,serr

      return

      end
      subroutine getscale2(calden,expden,imin,imax,jmin,jmax,kmin,kmax,
     &     shift,scale,calden2,maxmap,dim,dim2)

C----
C---- compute the optimal scaling parameters to be applied to the calculated
C---- densities in comparison to the experimental densities (in the least
C---- squares sense)
C----
      implicit none

      integer i,j,k,imin,imax,jmin,jmax,kmin,kmax,maxmap,ind,ind3,
     &     dim,dim2,ndata

      real calden(maxmap),expden(maxmap),shift,scale,rdum,
     &     calden2(maxmap)

      real*8 serr,avcal,avexp,c(2,2),d(2),e(2)

      avcal=0.0
      avexp=0.0
      c(1,1)=0.0
      c(1,2)=0.0
      c(2,1)=0.0
      c(2,2)=0.0
      ndata=0
      do k=kmin,kmax
         do j=jmin,jmax
            do i=imin,imax
               ind=ind3(i,j,k,dim,dim2)
               if (calden(ind).gt.1.0e-5) then
                  ndata=ndata+1
                  avexp=avexp+expden(ind)
                  avcal=avcal+calden2(ind)
                  c(1,1)=c(1,1)+expden(ind)**2                  
                  c(1,2)=c(1,2)+calden2(ind)*expden(ind)
                  c(2,2)=c(2,2)+calden2(ind)**2                  
                  c(2,1)=c(2,1)+calden2(ind)*expden(ind)
               endif
            enddo
         enddo
      enddo
      avexp=avexp/dble(ndata)
      avcal=avcal/dble(ndata)
      c(1,1)=c(1,1)/dble(ndata)-avexp**2
      c(1,2)=c(1,2)/dble(ndata)-avexp*avcal
      c(2,2)=c(2,2)/dble(ndata)-avcal**2
      c(2,1)=c(2,1)/dble(ndata)-avexp*avcal
C----
C---- diagonalise
C----
      call ql77(2,c,d,e,2)
c      write(6,'(f10.5)') c(1,2)/c(2,2)
c      write(0,'(a,2f10.5)') 'Eigenvalues:',d(1),d(2)
c      write(0,'(a,2f10.5)') 'Principal axis:',c(1,2),c(2,2)
C----
C---- determine scale and shift
C----
      scale=real(c(1,2)/c(2,2))
      shift=avexp-scale*avcal
      serr=0.0
      do k=kmin,kmax
         do j=jmin,jmax
            do i=imin,imax
               ind=ind3(i,j,k,dim,dim2)
               if (calden(ind).gt.1.0e-5) then
                  rdum=calden2(ind)*scale+shift
                  calden2(ind)=rdum
                  serr=serr+(expden(ind)-rdum)**2
               endif
            enddo
         enddo
      enddo

c      write(6,'(a,3e14.5)') 'Optimal shift,scale,error :',shift,
c     &     scale,serr

      return

      end

#     if defined (OPTIMISE_OCCUPANCIES)
      subroutine getocc(calden,expden,imin,imax,jmin,jmax,kmin,kmax,occ,
     &     maxat,maxmap,maxgrid,dim,dim2,ppi,ppj,ppk,ppc,np,nat,shift,
     &     scale)

      implicit none

      integer maxmap,maxat,maxgrid

      integer i,j,k,imin,imax,jmin,jmax,kmin,kmax,ind,ind3,mode(maxat),
     &     dim,dim2,ppi(*),ppj(*),ppk(*),np,
     &     iopt(10),n,iw(maxat*2),ret,nat

      real occ(*),calden(*),expden(*),rw(5*maxat),resid,
     &     ppc(maxgrid,*),lb(maxat),ub(maxat),shift,scale,maxocc
C----
C---- initialise
C----
      write(6,'(a)') 'Optimising occupancies...'
      maxocc=2.0
      do i=1,nat
         lb(i)=0.0
         ub(i)=maxocc
         mode(i)=3
         occ(i)=1.0
      enddo
      iopt(1)=99
C----
C---- set up the matrix: sbols requires the target in the last column
C----
      do n=1,np
         i=ppi(n)
         j=ppj(n)
         k=ppk(n)
         ind=ind3(i,j,k,dim,dim2)
         do i=1,nat
            ppc(n,i)=ppc(n,i)*scale+shift*(ppc(n,i)/calden(ind))
         enddo
         ppc(n,nat+1)=expden(ind)
      enddo
C----
C---- and solve
C----
      call sbols(ppc,maxgrid,np,nat,lb,ub,mode,iopt,occ,resid,ret,
     &     rw,iw)

c      write(6,'(a,i5)') 'sbols returned',ret
c      write(6,'(a,2e15.8)') 'residual:',resid,resid**2

      maxocc=0.0
      do i=1,nat
         maxocc=max(maxocc,occ(i))
      enddo
      scale=scale*maxocc
      shift=shift*maxocc
      do i=1,nat
         occ(i)=occ(i)/maxocc
         write(51,*) i,occ(i)
      enddo

      return

      end
#     endif 

      subroutine resoscat(reso,a1,a2,b1,b2,ntypes,maxtypes,fden,Bover)
C----
C---- modify atomic scattering factors due to limited resolution
C----
      implicit none

      integer i,j,h,maxref,maxden,ntypes,maxtypes

      parameter(maxref=1000,maxden=2000)

      real reso,a1(*),a2(*),b1(*),b2(*),R,rh,s,x,pi,frac,bb1,bb2,
     &     F(maxref),den(maxden),fden(maxtypes,*),Fsum,Bover

      pi=4.0*atan(1.0)
      frac=0.01

      do i=1,ntypes
C---- 
C---- calculate the true 1D density
C----
         bb1=b1(i)+Bover
         bb2=b2(i)+Bover
         do j=1,maxden
            x=(real(j)-1.0)/200.0
            den(j)=2.*a1(i)*sqrt(pi/bb1)*exp(-4.*pi*pi*x*x/bb1)+
     &           2.*a2(i)*sqrt(pi/bb2)*exp(-4.*pi*pi*x*x/bb2)
         enddo
C----
C---- calculate the atomic shape function
C----
         Fsum=0.0
         do h=1,maxref
            rh=(real(h)-1.0)*frac
            s=(rh**2)/4.0
            F(h)=a1(i)*exp(-bb1*s) + a2(i)*exp(-bb2*s)
            Fsum=Fsum+F(h)
         enddo
C----
C---- now back-fourier transform to density, applying resolution cutoff
C----
         do j=1,maxden
            x=(real(j)-1.0)/200.0
            fden(i,j)=F(1)
            do h=2,maxref
               rh=(real(h)-1.0)*frac
               R=1.0/sqrt(rh**2)
               if (R.gt.reso)
     &              fden(i,j)=fden(i,j)+2.*F(h)*cos(2.*pi*rh*x)
            enddo
C----
C---- and compare the density to the original density
C----
            fden(i,j)=fden(i,j)*frac
            write(20+i,*) x,den(j),fden(i,j)
         enddo
      enddo

      return

      end
      integer function ind3 (i, j, k, dim, dim2)
C----
C---- returns index a of 1-dimensional x(a) for the corresponding
C---- (equivalenced) 3-dimensional x(i,j,k)
C----
      implicit none
      
      integer i,j,k,dim,dim2

      ind3 = (k-1)*dim2+(j-1)*dim+i 

      return

      end

      subroutine readxplormap(name,nx,ny,nz,space,orig,den)
      
      implicit none

      integer i,j,k,ntit,nx,ny,nz,ngrid(3),orig(3),dim,dim2,den(*),
     &     gvflen

      real space(3),cell(6)

      character*255 buf

      character*(*) name

      logical gvfisf
C----
C---- read in experimental densities, Xplor formatted format
C----
      write(6,'(a)') 'Reading map...'
      if (gvfisf(name)) then
         open(1,file=name,status='old')
      else
         call gvstcp(name)
         write(0,'(a,a)') 'Error - file not found : ',
     &        name(1:gvflen(name))
         stop
      endif
      read(1,'(a255)') buf
      read(1,*) ntit
      do i=1,ntit
         read(1,'(a255)') buf
      enddo
      read(1,'(9i8)') 
     +  ngrid(1),orig(1),nx,
     +  ngrid(2),orig(2),ny,
     +  ngrid(3),orig(3),nz
      nx=nx-orig(1)+1
      ny=ny-orig(2)+1
      nz=nz-orig(3)+1
      read(1,'(1p,6e12.5)') (cell(i),i=1,6)
      if (cell(4).ne.90.0.or.cell(5).ne.90.0.or.cell(6).ne.90.0) then
         write(0,'(a)') 'Only orthogonal cells are supported. Sorry.'
         write(0,'(a)') 'Use e.g. the CCP4 program maprot to generate ',
     &        'one from your map.'

         stop
      endif
      space(1)=cell(1)/ngrid(1)
      space(2)=cell(2)/ngrid(2)
      space(3)=cell(3)/ngrid(3)
      read(1,'(a255)') buf
C----
C---- this trick allows den to be 1-dimensional and therefore we
C---- can be more memory-efficient (faster too!)
C---- den(i,j,k) is now den((k-1)*dim2+(j-1)*dim+i)
C---- (this would btw also be the result of equivalencing the two)
C----
      dim=nx
      dim2=nx*ny
      do k=1,nz
         read(1,'(a255)') buf
         read(1,'(6e12.5)') ((den((k-1)*dim2+(j-1)*dim+i),
     &        i=1,nx),j=1,ny)
      enddo
      close(1)

      return

      end
      subroutine readxplormapheader(name,cell,nx,ny,nz,space,orig,ngrid)
      
      implicit none

      integer i,ntit,nx,ny,nz,ngrid(3),orig(3),dim,dim2,gvflen

      real space(3),cell(6)

      logical gvfisf

      character*255 buf
      character*(*) name
C----
C---- read in map header, Xplor formatted format
C----
      write(6,'(a)') 'Reading map header...'
      if (gvfisf(name)) then
         open(1,file=name,status='old')
      else
         call gvstcp(name)
         write(0,'(a,a)') 'Error - file not found : ',
     &        name(1:gvflen(name))
         stop
      endif
      read(1,'(a255)') buf
      read(1,*) ntit
      do i=1,ntit
         read(1,'(a255)') buf
      enddo
      read(1,'(9i8)') 
     +  ngrid(1),orig(1),nx,
     +  ngrid(2),orig(2),ny,
     +  ngrid(3),orig(3),nz
      nx=nx-orig(1)+1
      ny=ny-orig(2)+1
      nz=nz-orig(3)+1
      read(1,'(1p,6e12.5)') (cell(i),i=1,6)
      if (cell(4).ne.90.0.or.cell(5).ne.90.0.or.cell(6).ne.90.0) then
         write(0,'(a)') 'Only orthogonal cells are supported. Sorry.'
         write(0,'(a)') 'Use e.g. the CCP4 program maprot to generate ',
     &        'one from your map.'
         stop
      endif
      space(1)=cell(1)/ngrid(1)
      space(2)=cell(2)/ngrid(2)
      space(3)=cell(3)/ngrid(3)
      write(0,'(a)') 'Map parameters:'
      write(0,'(a,3f8.3)') 'Cell axes  :',(cell(i),i=1,3)
      write(0,'(a,3f8.3)') 'Cell angles:',(cell(i),i=4,6)
      write(0,'(a,3i8)') 'Grid       :',(ngrid(i),i=1,3)
      write(0,'(a,3i8)') 'Extent     :',nx,ny,nz
      write(0,'(a,3i8)') 'Origin     :',(orig(i),i=1,3)
      write(0,'(a,3f8.3)') 'Spacing    :',(space(i),i=1,3)

      close(1)

      return

      end

      SUBROUTINE QL77 (N,X,D,E,NMAX)
C
C     MATRIX DIAGONALIZATION ROUTINE
C
C     CODE DERIVED FROM EISPACK
C     UPDATE : FMP 16.2.86 (CONVERSION TO FORTRAN 77)
C     UPDATE : FMP 09.3.89 (get rid of double precision)
C
C
C     INPUT
C
C     N           ORDER OF THE MATRIX, NOT LARGER THAN NMAX
C     X(N,N)      THE REAL SYMMETRIC MATRIX TO BE DIAGONALIZED AS A
C                 TWO-INDEXED SQUARE ARRAY OF WHICH ONLY THE LOWER LEFT
C                 TRIANGLE IS USED AND NEED BE SUPPLIED
C     NMAX        MAXIMUM VALUE OF N AS DECLARED IN THE CALLING ROUTINE
C                 DIMENSION STATEMENT
C     E           WORKING SPACE OF LENGTH NMAX
C
C     OUTPUT
C
C     D(N)        COMPUTED EIGENVALUES IN DESCENDING ORDER 
C     X(N,N)      EIGENVECTORS (IN COLUMNS) IN THE SAME ORDER AS THE
C                 EIGENVALUES
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)

      INTEGER N,NMAX,NI,L,K,J,J1,I

      REAL*8  X(NMAX,NMAX), D(NMAX), E(NMAX), EPS, TOL
      DATA EPS, TOL/ 7.E-14, 1.E-45/
C
C     HANDLE SPECIAL CASE                                               
C
      IF (N.NE.1) GOTO 10
      D(1)=X(1,1)
      X(1,1)=1.0
      RETURN
C
C     HOUSEHOLDER REDUCTION TO TRIDIAGONAL FORM                         
C
   10 DO 150 NI=2,N
      I=N+2-NI
      L=I-2
      H=0.0                                                           
      G=X(I,I-1)
      IF (L.le.0)goto 140
      DO 30 K=1,L
   30 H=H+X(I,K)**2                                                     
      S=H+G*G                                                           
      IF(S.GE.TOL) GOTO 50
      H=0.0                                                           
      GOTO 140
   50 IF (H.le.0.0)goto 140
      L=L+1
      F=G
      G=SQRT(S)
      IF (F.le.0.0)goto 70
   70 G=-G
      H=S-F*G                                                           
      X(I,I-1)=F-G
      F=0.0
      DO 110 J=1,L
      X(J,I)=X(I,J)/H
      S=0.0
      DO 80 K=1,J
   80 S=S+X(J,K)*X(I,K)
      J1=J+1
      IF (J1.GT.L) GOTO 100
      DO 90 K=J1,L
   90 S=S+X(K,J)*X(I,K)
  100 E(J)=S/H
  110 F=F+S*X(J,I)
      F=F/(H+H)
      DO 120 J=1,L
  120 E(J)=E(J)-F*X(I,J)
      DO 130 J=1,L
      F=X(I,J)
      S=E(J)
      DO 130 K=1,J
  130 X(J,K)=X(J,K)-F*E(K)-X(I,K)*S
  140 D(I)=H
  150 E(I-1)=G
C
C     ACCUMULATION OF TRANSFORMATION MATRIX AND INTERMEDIATE D VECTOR
C
      D(1)=X(1,1)
      X(1,1)=1.0
      DO 220 I=2,N
      L=I-1
      IF (D(I).le.0.0)goto 200
      DO 190 J=1,L
      S=0.0
      DO 180 K=1,L
  180 S=S+X(I,K)*X(K,J)
      DO 190 K=1,L
  190 X(K,J)=X(K,J)-S*X(K,I)
  200 D(I)=X(I,I)
      X(I,I)=1.0
      DO 220 J=1,L
      X(I,J)=0.0
  220 X(J,I)=0.0
C
C     QL ITERATES
C
      B=0.0
      F=0.0
      E(N)=0.0
      DO 340 L=1,N
      H=EPS*(DABS(D(L))+DABS(E(L)))                                     
      IF (H.GT.B) B=H                                                   
      DO 240 J=L,N
      IF(DABS(E(J)).LE.B) GOTO 250
  240 CONTINUE
  250 IF(J.EQ.L) GOTO 340
  260 G=D(L)
      P=(D(L+1)-G)*0.5/E(L)
      R=SQRT(P*P+1.0)
      IF(P.ge.0.0)goto 280
      P=P-R
      GOTO 290
  280 P=P+R
  290 D(L)=E(L)/P
      H=G-D(L)                                                       
      K=L+1
      DO 300 I=K,N
  300 D(I)=D(I)-H                                                       
      F=F+H                                                             
      P=D(J)
      C=1.0
      S=0.0
      J1=J-1
      DO 330 NI=L,J1
      I=L+J1-NI
      G=C*E(I)
      H=C*P
      IF(DABS(P).LT.DABS(E(I))) GOTO 310
      C=E(I)/P
      R=SQRT(C*C+1.0)
      E(I+1)=S*P*R
      S=C/R
      C=1.0/R
      GOTO 320
  310 C=P/E(I)
      R=SQRT(C*C+1.0)
      E(I+1)=S*E(I)*R
      S=1.0/R
      C=C/R
  320 P=C*D(I)-S*G
      D(I+1)=H+S*(C*G+S*D(I))
      DO 330 K=1,N
      H=X(K,I+1)
      X(K,I+1)=X(K,I)*S+H*C
  330 X(K,I)=X(K,I)*C-H*S
      E(L)=S*P
      D(L)=C*P
      IF(DABS(E(L)).GT.B) GOTO 260
  340 D(L)=D(L)+F

C
C**** PUT EIGENVALUES AND EIGENVECTORS IN 
C**** DESIRED ASCENDING ORDER
C

      NI = N-1

      DO I = 1,NI

         K    = I
         P    = D(I)
         ABSP = DABS(D(I))
         J1   = I+1

         DO J = J1,N
            IF(DABS(D(J)) .LT. ABSP) THEN
               K    = J
               P    = D(J)
               ABSP = DABS(D(J))
            ENDIF
         ENDDO

         IF (K. NE. I) THEN
            D(K) = D(I)
            D(I) = P
            DO J = 1,N
               P      = X(J,I)
               X(J,I) = X(J,K)
               X(J,K) = P
            ENDDO
         ENDIF

      ENDDO

      RETURN

C
C     FMP
C     LAST BUT NOT LEAST I HAVE TO CONFESS THAT THERE IS AN ORIGINAL    
C     G. BINSCH REMARK ON THIS ROUTINE: 'QL IS PURPORTED TO BE ONE OF
C     THE FASTEST AND MOST COMPACT ROUTINES OF ITS KIND PRESENTLY KNOWN
C     TO MANKIND.'
C
      END
