      program pdb2map
C----
C---- (C) Bert de Groot, Goettingen 2000
C----
C---- creates a density map from a coordinate file
C----
      implicit none

      include 'params.f'

      integer i,j,k,nat,atnr(maxat),resnr(maxat),nres,gvflen,
     &     nx,ny,nz,ngrid(3),orig(3),ntit,id(maxat),iargc,
     &     dim,dim2
      
      character*5 atnam(maxat),resnam(maxat)

      character*4 cdum1,cdum2

      character*1 chainid(maxat)

      real pi,space(3),cell(6),xx(maxat),yy(maxat),zz(maxat),
     &     calden(maxmap),den1(maxat),den2(maxat),a1(10),Bfac(maxat),
     &     a2(10),b1(10),b2(10),occ(maxat)

      character*80 sdum

      character*255 buf

      logical gvfisf
C----
C---- initialise
C----
      if (iargc().ne.3) then
         write(0,'(a)') 'Usage: pdb2map in.map in.pdb out.map'
         write(0,*) 
         write(0,'(a,a)') 'Generates a map "out.map" based on "in.pdb"',
     &        ', using the map parameters from "in.map".'
         write(0,'(a,a)') '"in.map" has to be x-plor formatted format',
     &        ' for now.'
         stop
      endif
      pi=4.0*atan(1.0)
C----
C---- read map header for params
C----
      call getarg(1,sdum)
      call getarg(1,sdum)
      call readxplormapheader(sdum,cell,nx,ny,nz,space,orig,ngrid)
      if (nx*ny*nz.gt.maxmap) then
         write(0,'(a)') 'Error - nx,ny,nz too large.'
         write(0,'(a,i10)') 'Increase maxmap to at least',nx*ny*nz
         stop
      endif
C----
C---- read input structure
C----
      call getarg(2,sdum)
      if (gvfisf(sdum)) then
         open(1,file=sdum,status='old')
      else
         call gvstcp(sdum)
         write(0,'(a,a)') 'Error - file not found : ',
     &        sdum(1:gvflen(sdum))
         stop
      endif
      nat=0
      nres=1
      cdum2='xxxx'
 50   continue
      read(1,'(a80)',err=60,end=70) sdum
      if ((sdum(1:4).ne.'ATOM').and.(sdum(1:4).ne.'HETA')) goto 50
      nat=nat+1
      if (nat.gt.maxat) then
         write(0,'(a)') 'Error - MAXAT exceeded'
         stop
      endif
      read(sdum,2000) atnam(nat),resnam(nat),chainid(nat),cdum1,
     &     xx(nat),yy(nat),zz(nat),occ(nat),Bfac(nat)
      if (nat.gt.1) then
         if (cdum1.ne.cdum2) nres=nres+1
      endif
      if (atnam(nat).ne.'     ') then
         call gvsshr(atnam(nat),5)
      else
         write(0,*) 'Encountered empty atom name for atom number',nat
         stop
      endif
      resnr(nat)=nres
      atnr(nat)=nat
      cdum2=cdum1
      goto 50
 60   write(0,'(a)') 'Error while reading input structure'
      stop
 70   continue
      if (nat.eq.0) then
         write(0,'(a,a)') 'Unexpected end of file while reading ',
     &        'input structure'
         stop
      endif
      write(6,'(a,i5)') 'Nr of atoms:',nat
      close(1)
C----
C---- get the atomic scattering factors etc
C----
      call getatomscat(a1,a2,b1,b2,atnam,nat,Bfac,id,den1,den2)
      do i=1,nat
         den1(i)=den1(i)*occ(i)
         den2(i)=den2(i)*occ(i)
      enddo
C----
C---- calculate the densities
C----
      call calc_dens(den1,den2,id,a1,a2,b1,b2,xx,yy,zz,Bfac,nat,
     &     calden,space,orig,nx,ny,nz,maxat,maxmap)
C----
C---- write out the map
C----
      write(6,'(a)') 'Writing output map...'
      call getarg(3,sdum)
      open(1,file=sdum,status='unknown')
      write(1,*)
      write(1,'(i8)') 1
      write(1,'(a)') 'REMARKS map generated by pdb2map'
      write(1,'(9i8)') 
     &  ngrid(1),orig(1),nx+orig(1)-1,
     &  ngrid(2),orig(2),ny+orig(2)-1,
     &  ngrid(3),orig(3),nz+orig(3)-1
      write(1,'(1p,6e12.5)') (cell(i),i=1,6)
      write(1,'(a)') 'ZYX'
      dim=nx
      dim2=nx*ny
      do k=1,nz
         write(1,'(i8)') k
         write(1,'(6e12.5)') ((calden((k-1)*dim2+(j-1)*dim+i),
     &        i=1,nx),j=1,ny)
      enddo
      close(1)

 2000 format (12x,a4,1x,a4,a1,a4,4x,3f8.3,2f6.2)

      end
      
